@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import asyncio
 import os
-from dataclasses import dataclass
+from dataclasses import dataclass, replace
 from typing import TYPE_CHECKING, Any, TypedDict
 
 import aiohttp
@@ -30,8 +30,6 @@ from temporalio.runtime import (
 )
 from temporalio.worker import (
     PollerBehaviorAutoscaling,
-    ResourceBasedSlotConfig,
-    ResourceBasedSlotSupplier,
     ResourceBasedTunerConfig,
     Worker,
     WorkerTuner,
@@ -40,6 +38,7 @@ from typing_extensions import Unpack
 
 from .endpoints import explore_class_details
 from .observability import log_with_context
+from .payload_codec import LargePayloadCodec
 from .playground.workflow import FunctionsOnly, PlaygroundRun
 
 if TYPE_CHECKING:
@@ -59,6 +58,7 @@ class CloudConnectionOptions:
     api_key: str
     address: str | None = "localhost:7233"
     api_address: str | None = None
+    codec_address: str | None = None
     temporal_namespace: str | None = "default"
     data_converter: DataConverter | None = None
 
@@ -74,9 +74,8 @@ class ServiceOptions:
 
 @dataclass
 class ResourceOptions:
-    slots: int | None = 10
-    target_cpu_usage: float | None = 0.8
-    target_memory_usage: float | None = 0.8
+    target_cpu_usage: float | None = None
+    target_memory_usage: float | None = None
 
 
 class Restack:
@@ -124,6 +123,11 @@ class Restack:
             and self.options.api_address is not None
             else "http://localhost:6233"
         )
+        codec_address = (
+            f"https://{os.getenv('RESTACK_ENGINE_API_ADDRESS')}"
+            if os.getenv("RESTACK_ENGINE_API_ADDRESS")
+            else "http://localhost:12233"
+        )
         engine_id = (
             self.options.engine_id
             if self.options and self.options.engine_id is not None
@@ -134,6 +138,7 @@ class Restack:
             "metadata": {
                 "restack-engineId": engine_id,
                 "restack-apiAddress": api_address,
+                "restack-codecAddress": codec_address,
             },
         }
         if self.options and self.options.api_key is not None:
@@ -195,12 +200,6 @@ class Restack:
         )
         tls = bool(self.options and self.options.api_key)
         metadata = connect_options["metadata"]
-        data_converter = (
-            self.options.data_converter
-            if self.options
-            and self.options.data_converter is not None
-            else pydantic_data_converter
-        )
 
         if (
             os.getenv("RESTACK_TELEMETRY_OTEL_ADDRESS")
@@ -219,13 +218,19 @@ class Restack:
             runtime = Runtime(telemetry=TelemetryConfig())
 
         try:
+            api_address = connect_options["metadata"]["restack-codecAddress"]
             client = await Client.connect(
                 target_host,
                 namespace=namespace,
                 api_key=api_key,
                 tls=tls,
                 rpc_metadata=metadata,
-                data_converter=data_converter,
+                data_converter=replace(
+                    pydantic_data_converter, payload_codec=LargePayloadCodec(
+                        api_address,
+                        namespace,
+                    ),
+                ),
                 runtime=runtime,
             )
         except Exception as e:
@@ -275,19 +280,9 @@ class Restack:
                     resources.target_cpu_usage or 0.8,
                     resources.target_memory_usage or 0.8,
                 )
-                tuner = WorkerTuner.create_composite(
-                    workflow_supplier=ResourceBasedSlotSupplier(
-                        ResourceBasedSlotConfig(),
-                        resource_based_options,
-                    ),
-                    activity_supplier=ResourceBasedSlotSupplier(
-                        ResourceBasedSlotConfig(),
-                        resource_based_options,
-                    ),
-                    local_activity_supplier=ResourceBasedSlotSupplier(
-                        ResourceBasedSlotConfig(),
-                        resource_based_options,
-                    ),
+                tuner = WorkerTuner.create_resource_based(
+                    target_cpu_usage=resource_based_options.target_cpu_usage,
+                    target_memory_usage=resource_based_options.target_memory_usage,
                 )
 
                 service = Worker(
@@ -432,6 +427,7 @@ class Restack:
         functions: list[Any]
         task_queue: str
         options: ServiceOptions
+        resources: ResourceOptions
 
     async def start_service(
         self,
@@ -459,7 +455,7 @@ class Restack:
                 - functions (list[Function] | None): A list of functions to execute.
                 - task_queue (str | None): The name of the task queue.
                 - options (ServiceOptions | None): Additional service options.
-
+                - resources (ResourceOptions | None): Resource options.
         """
         workflows = kwargs.get("workflows", [])
         agents = kwargs.get("agents", [])
@@ -626,7 +622,7 @@ class Restack:
     async def get_workflow_handle(
         self,
         workflow_id: str,
-        run_id: str,
+        run_id: str | None = None,
     ) -> Any:
         await self.connect()
         if self.client:
@@ -658,7 +654,7 @@ class Restack:
     async def get_workflow_result(
         self,
         workflow_id: str,
-        run_id: str,
+        run_id: str | None = None,
     ) -> Any:
         handle = await self.get_workflow_handle(
             workflow_id,
@@ -705,22 +701,22 @@ class Restack:
     async def get_agent_handle(
         self,
         agent_id: str,
-        run_id: str,
+        run_id: str | None = None,
     ) -> Any:
         return await self.get_workflow_handle(agent_id, run_id)
 
     async def get_agent_result(
         self,
         agent_id: str,
-        run_id: str,
+        run_id: str | None = None,
     ) -> Any:
         return await self.get_workflow_result(agent_id, run_id)
 
     async def get_agent_state(
         self,
         agent_id: str,
-        run_id: str,
         state_name: str,
+        run_id: str | None = None,
     ) -> Any:
         handle = await self.get_workflow_handle(agent_id, run_id)
         try:
